/*	$CoreSDI: log.h,v 1.12 2001/08/03 00:30:19 claudio Exp $	*/

/*
 *   Copyright (c) 2000, 2001, Core SDI S.A., Argentina
 *   All rights reserved
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *   3. Neither name of the Core SDI S.A. nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *  
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 *   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *   IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *   THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef LOG_H
#define LOG_H

typedef enum {
	LOGF_NOLOG	= 0x01,	/* Do not log any messages		*/
	LOGF_SESSION	= 0x02,	/* Log session messages			*/
	LOGF_DEBUG	= 0x04,	/* Log debugging messages		*/
	LOGF_ERROR	= 0x08,	/* Log warning, error and fatal messages*/
	LOGF_STDERR	= 0x10  /* Log on stderr instead of syslog(3)	*/
} LOGFLAGS;

void	log_init (LOGFLAGS, int);
void	log_msg (int, const char *, ...);
void	log_session (const char *, ...);
void	log_cmd (const char *, const char *, const char *, ...);
void	log_vcmd (const char *, const char *, const char *, va_list);
void	log_debug (const char *, ...);
void	log_warn (const char *, ...);
void	log_err (const char *, ...);
void	log_fatal (int, const char *, ...);
int	cleanup_add (void (*) (void *), void *);
void	cleanup_rm_all ();

#define fatal	log_fatal

#endif /* LOG_H */

